]> sigrok.org Git - libsigrok.git/blobdiff - session.c
sr: Various fixes in the udev file.
[libsigrok.git] / session.c
index e129da9efd76db9673ff8c1b750c663f69c2c5ac..8048fdfd7b22e00dc3294395076d23b8672a0dd4 100644 (file)
--- a/session.c
+++ b/session.c
@@ -32,8 +32,8 @@ struct source {
        int fd;
        int events;
        int timeout;
-       sr_receive_data_callback cb;
-       void *user_data;
+       sr_receive_data_callback_t cb;
+       void *cb_data;
 };
 
 /* There can only be one session at a time. */
@@ -114,7 +114,7 @@ SR_API int sr_session_dev_clear(void)
  * Add a device to the current session.
  *
  * @param dev The device to add to the current session. Must not be NULL.
*            Also, dev->plugin and dev->plugin->opendev must not be NULL.
  *
  * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments.
  */
@@ -127,24 +127,29 @@ SR_API int sr_session_dev_add(struct sr_dev *dev)
                return SR_ERR_ARG;
        }
 
-       if (!dev->plugin) {
-               sr_err("session: %s: dev->plugin was NULL", __func__);
-               return SR_ERR_ARG;
+       if (!session) {
+               sr_err("session: %s: session was NULL", __func__);
+               return SR_ERR_BUG;
        }
 
-       if (!dev->plugin->opendev) {
-               sr_err("session: %s: dev->plugin->opendev was NULL",
-                      __func__);
-               return SR_ERR_ARG;
+       /* If dev->driver is NULL, this is a virtual device. */
+       if (!dev->driver) {
+               sr_dbg("session: %s: dev->driver was NULL, this seems to be "
+                      "a virtual device; continuing", __func__);
+               /* Just add the device, don't run dev_open(). */
+               session->devs = g_slist_append(session->devs, dev);
+               return SR_OK;
        }
 
-       if (!session) {
-               sr_err("session: %s: session was NULL", __func__);
-               return SR_ERR; /* TODO: SR_ERR_BUG? */
+       /* dev->driver is non-NULL (i.e. we have a real device). */
+       if (!dev->driver->dev_open) {
+               sr_err("session: %s: dev->driver->dev_open was NULL",
+                      __func__);
+               return SR_ERR_BUG;
        }
 
-       if ((ret = dev->plugin->opendev(dev->plugin_index)) != SR_OK) {
-               sr_err("session: %s: opendev failed (%d)", __func__, ret);
+       if ((ret = dev->driver->dev_open(dev->driver_index)) != SR_OK) {
+               sr_err("session: %s: dev_open failed (%d)", __func__, ret);
                return ret;
        }
 
@@ -174,21 +179,25 @@ SR_API int sr_session_datafeed_callback_clear(void)
 /**
  * Add a datafeed callback to the current session.
  *
- * @param callback Function to call when a chunk of data is received.
+ * @param cb Function to call when a chunk of data is received.
+ *           Must not be NULL.
  *
  * @return SR_OK upon success, SR_ERR_BUG if no session exists.
  */
-SR_API int sr_session_datafeed_callback_add(sr_datafeed_callback callback)
+SR_API int sr_session_datafeed_callback_add(sr_datafeed_callback_t cb)
 {
        if (!session) {
                sr_err("session: %s: session was NULL", __func__);
                return SR_ERR_BUG;
        }
 
-       /* TODO: Is 'callback' allowed to be NULL? */
+       if (!cb) {
+               sr_err("session: %s: cb was NULL", __func__);
+               return SR_ERR_ARG;
+       }
 
        session->datafeed_callbacks =
-           g_slist_append(session->datafeed_callbacks, callback);
+           g_slist_append(session->datafeed_callbacks, cb);
 
        return SR_OK;
 }
@@ -233,7 +242,7 @@ static int sr_session_run_poll(void)
                                 * asked for that timeout.
                                 */
                                if (!sources[i].cb(fds[i].fd, fds[i].revents,
-                                                 sources[i].user_data))
+                                                 sources[i].cb_data))
                                        sr_session_source_remove(sources[i].fd);
                        }
                }
@@ -259,25 +268,25 @@ SR_API int sr_session_start(void)
        if (!session) {
                sr_err("session: %s: session was NULL; a session must be "
                       "created first, before starting it.", __func__);
-               return SR_ERR; /* TODO: SR_ERR_BUG? */
+               return SR_ERR_BUG;
        }
 
        if (!session->devs) {
                /* TODO: Actually the case? */
                sr_err("session: %s: session->devs was NULL; a session "
                       "cannot be started without devices.", __func__);
-               return SR_ERR; /* TODO: SR_ERR_BUG? */
+               return SR_ERR_BUG;
        }
 
-       /* TODO: Check plugin_index validity? */
+       /* TODO: Check driver_index validity? */
 
        sr_info("session: starting");
 
        for (l = session->devs; l; l = l->next) {
                dev = l->data;
                /* TODO: Check for dev != NULL. */
-               if ((ret = dev->plugin->acquisition_start(
-                               dev->plugin_index, dev)) != SR_OK) {
+               if ((ret = dev->driver->dev_acquisition_start(
+                               dev->driver_index, dev)) != SR_OK) {
                        sr_err("session: %s: could not start an acquisition "
                               "(%d)", __func__, ret);
                        break;
@@ -318,7 +327,7 @@ SR_API int sr_session_run(void)
        if (num_sources == 1 && sources[0].fd == -1) {
                /* Dummy source, freewheel over it. */
                while (session->running)
-                       sources[0].cb(-1, 0, sources[0].user_data);
+                       sources[0].cb(-1, 0, sources[0].cb_data);
        } else {
                /* Real sources, use g_poll() main loop. */
                sr_session_run_poll();
@@ -352,7 +361,7 @@ SR_API int sr_session_halt(void)
  * Stop the current session.
  *
  * The current session is stopped immediately, with all acquisition sessions
- * being stopped and hardware plugins cleaned up.
+ * being stopped and hardware drivers cleaned up.
  *
  * @return SR_OK upon success, SR_ERR_BUG if no session exists.
  */
@@ -372,11 +381,11 @@ SR_API int sr_session_stop(void)
        for (l = session->devs; l; l = l->next) {
                dev = l->data;
                /* Check for dev != NULL. */
-               if (dev->plugin) {
-                       if (dev->plugin->acquisition_stop)
-                               dev->plugin->acquisition_stop(dev->plugin_index, dev);
-                       if (dev->plugin->cleanup)
-                               dev->plugin->cleanup();
+               if (dev->driver) {
+                       if (dev->driver->dev_acquisition_stop)
+                               dev->driver->dev_acquisition_stop(dev->driver_index, dev);
+                       if (dev->driver->cleanup)
+                               dev->driver->cleanup();
                }
        }
 
@@ -419,26 +428,21 @@ static void datafeed_dump(struct sr_datafeed_packet *packet)
  * Hardware drivers use this to send a data packet to the frontend.
  *
  * @param dev TODO.
- * @param packet TODO.
+ * @param packet The datafeed packet to send to the session bus.
  *
  * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments.
  */
-SR_PRIV int sr_session_bus(struct sr_dev *dev,
-                          struct sr_datafeed_packet *packet)
+SR_PRIV int sr_session_send(struct sr_dev *dev,
+                           struct sr_datafeed_packet *packet)
 {
        GSList *l;
-       sr_datafeed_callback cb;
+       sr_datafeed_callback_t cb;
 
        if (!dev) {
                sr_err("session: %s: dev was NULL", __func__);
                return SR_ERR_ARG;
        }
 
-       if (!dev->plugin) {
-               sr_err("session: %s: dev->plugin was NULL", __func__);
-               return SR_ERR_ARG;
-       }
-
        if (!packet) {
                sr_err("session: %s: packet was NULL", __func__);
                return SR_ERR_ARG;
@@ -463,23 +467,23 @@ SR_PRIV int sr_session_bus(struct sr_dev *dev,
  * @param fd TODO.
  * @param events TODO.
  * @param timeout TODO.
* @param callback TODO.
* @param user_data TODO.
  *
  * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments, or
  *         SR_ERR_MALLOC upon memory allocation errors.
  */
 SR_API int sr_session_source_add(int fd, int events, int timeout,
-               sr_receive_data_callback callback, void *user_data)
+               sr_receive_data_callback_t cb, void *cb_data)
 {
        struct source *new_sources, *s;
 
-       if (!callback) {
-               sr_err("session: %s: callback was NULL", __func__);
+       if (!cb) {
+               sr_err("session: %s: cb was NULL", __func__);
                return SR_ERR_ARG;
        }
 
/* Note: user_data can be NULL, that's not a bug. */
 
        new_sources = g_try_malloc0(sizeof(struct source) * (num_sources + 1));
        if (!new_sources) {
@@ -497,8 +501,8 @@ SR_API int sr_session_source_add(int fd, int events, int timeout,
        s->fd = fd;
        s->events = events;
        s->timeout = timeout;
-       s->cb = callback;
-       s->user_data = user_data;
+       s->cb = cb;
+       s->cb_data = cb_data;
        sources = new_sources;
 
        if (timeout != source_timeout && timeout > 0
@@ -526,7 +530,7 @@ SR_API int sr_session_source_remove(int fd)
 
        if (!sources) {
                sr_err("session: %s: sources was NULL", __func__);
-               return SR_ERR_BUG; /* TODO: Other? */
+               return SR_ERR_BUG;
        }
 
        /* TODO: Check if 'fd' valid. */
@@ -114,7 +114,7 @@ SR_API int sr_session_dev_clear(void)
  *
  * @param dev The device to add to the current session. Must not be NULL.
- *            Also, dev->plugin and dev->plugin->opendev must not be NULL.
+ *            Also, dev->driver and dev->driver->dev_open must not be NULL.
  *
  * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments.
@@ -463,23 +467,23 @@ SR_PRIV int sr_session_bus(struct sr_dev *dev,
  * @param events TODO.
  * @param timeout TODO.
- * @param callback TODO.
- * @param user_data TODO.
+ * @param cb Callback function to add. Must not be NULL.
+ * @param cb_data Data for the callback function. Can be NULL.
  *
  * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments, or
@@ -463,23 +467,23 @@ SR_PRIV int sr_session_bus(struct sr_dev *dev,
        }
 
-       /* Note: user_data can be NULL, that's not a bug. */
+       /* Note: cb_data can be NULL, that's not a bug. */
 
        new_sources = g_try_malloc0(sizeof(struct source) * (num_sources + 1));